"""
Chestnut Scientific Library
Copyright (C) 2008, Damian Eads. All Rights Reserved.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

#from setuptools import setup, Extension
from distutils.core import setup, Extension

import sys, os, os.path, string

def contains_arrayobject_h(path):
    """
    Returns True iff the python path string contains the arrayobject.h
    include file where it is supposed to be.
    """
    f=False
    try:
        s=os.stat(os.path.join(path, 'numpy', 'core', 'include', \
                               'numpy', 'arrayobject.h'))
        f=True
    except OSError:
        pass
    return f

if sys.platform != 'darwin':
    extra_link_args = ['-s']

valid_paths = filter(contains_arrayobject_h, sys.path)
if len(valid_paths) == 0:
    print "No paths in the python path contain numpy/arrayobject.h"
    sys.exit(0)

# The base path is by default the first python path with arrayobject.h in it.
include_numpy_array=valid_paths[0]

if len(valid_paths) > 1:
    print "There are several valid include directories containing numpy/arrayobject.h"
    l=[('%d: %s' % (i+1, valid_paths[i])) for i in xrange(0, len(valid_paths))]
    s = -1
    print string.join(l, '\n')
    # Prompt the user with a list of selections.
    while not (s >= 1 and s <= len(valid_paths)):
        s = input('Selection [default=1]:' % s)
        if s == '':
            s = 1
        else:
            s = int(s)
    include_numpy_array=valid_paths[s-1]

# Add the children directory path suffix to the base path.
include_numpy_array=os.path.join(include_numpy_array, 'numpy', 'core', \
                                 'include')

class Toolkit:

    def __init__(self, name, exts):
        """
        Creates a new Toolkit object.

          - name: The name of the toolkit (e.g. "kmeans")
          - exts: A list of extension modules for the toolkit.
        """
        self.name = name
        self.exts = exts

extra_link_args = []

#####################################################################
#                                                                   #
#                       Toolkit Definitions                         #
#                                                                   #
#####################################################################

########################### hcluster ################################
 
cluster_ext = Extension('_cluster_wrap',
                         ['chestnut/cluster/cluster.c',
                          'chestnut/cluster/cluster_wrap.cpp'],
                         extra_link_args = extra_link_args,
                         include_dirs=['cluster/',
                                       include_numpy_array])

cluster = Toolkit(name="cluster",
                  exts=[cluster_ext])

############################# hmm ###################################

hmm = Toolkit(name="hmm", exts=[])

########################### boosting ################################

boosting = Toolkit(name="boosting", exts=[])

############################# dtw ###################################

dtw = Toolkit(name="dtw", exts=[])

# END END END END END END END END END END END END END END END END END
################### End of Toolkit Definitions ######################

released_toolkits = [cluster]
devel_toolkits = [cluster, hmm, boosting, dtw]

if len(sys.argv) > 1 and sys.argv[1] == 'devel':
    setup_args = sys.argv[2:]
    devel = True
else:
    setup_args = sys.argv[1:]
    devel = False

#####################################################################
#                                                                   #
#                     Now let's build everything                    #
#                                                                   #
#####################################################################

ext_modules = []
chestnut_packages = []
if devel:
    for toolkit in devel_toolkits:
        ext_modules += toolkit.exts
        chestnut_packages.append("chestnut." + toolkit.name)
else:
    for toolkit in released_toolkits:
        ext_modules += toolkit.exts
        chestnut_packages.append("chestnut." + toolkit.name)

chestnut_description="""
A collection of data mining and machine learning toolkits for Python.
""".strip()

setup(name='chestnut',
      version='0.1',
      description=chestnut_description,
      long_description=chestnut_description,
      ext_modules=ext_modules,
      author="Damian Eads, Ed Rosten, Andy Fraser, Karen Glocer",
      author_email="eads@soe.ucsc.edu",
      license="GNU General Public License (GPL)",
      packages = chestnut_packages,
      url = 'http://www.chestnut.org',
      script_args = setup_args,
      exclude_package_data = {'': 'COPYING'}
      )

